#!/usr/bin/env bash

# GENERIC B5 FUNCTIONS

B5_FONT_GREEN='\033[00;32m'
B5_FONT_RED='\033[00;31m'
B5_FONT_RESTORE='\033[0m'

b5_init() {
    local directory="$1"
    local template="$2"

    if [ -z "${directory}" ]
    then
        echo "No directory given. Abort task run."
        echo "Usage: b5_init <directory> [template]"
        return 1
    fi

    if [ -z "${template}" ]
    then
        template='minimal'
    fi

    if [ -d "${directory}" ]
    then
        echo "Directory already exists. Aborting!"
        return 1
    fi

    if [ ! -d "${TEMPLATES_PATH}/${template}" ]
    then
        echo "Template does not exist. Aborting!"
        return 1
    fi

    (
        mkdir -p "${directory}" && \
        cp -r "${TEMPLATES_PATH}/${template}"/* "${directory}"/ && \
        cd "${directory}" && \
        git init . && \
        echo "All set up. You may now start using ${directory}"
    )
}

b5_abort() {
    echo -e "${B5_FONT_RED}$1${B5_FONT_RESTORE}"
    exit 1
}

b5_run() {
    "$@" && \
        echo -e "\n${B5_FONT_GREEN}Task exited ok${B5_FONT_RESTORE}" ||  \
        echo -e "\n${B5_FONT_RED}Task failed, see error above${B5_FONT_RESTORE}"
}

b5_help() {
    echo "$0 <task> <args>"
    echo "Tasks:"
    compgen -A function | sed -En 's/task:(.*)/\1/p' | cat -n
    echo
    echo "Debug for development:"
    config_dump
}

b5_install() {
    true  # nothing to do
}

b5_update() {
    true  # nothing to do
}

# YAML (See https://github.com/jasperes/bash-yaml)

yaml_parse() {
    local yaml_file=$1
    local prefix=$2
    local s
    local w
    local fs

    s='[[:space:]]*'
    w='[a-zA-Z0-9_]*'
    fs="$(echo @|tr @ '\034')"

    (
        sed -ne 's/--//g; s/\"/\\\"/g; s/\#.*//g; s/\s*$//g;' \
            -e  "s|^\($s\)\($w\)$s:$s\"\(.*\)\"$s\$|\1$fs\2$fs\3|p" \
            -e "s|^\($s\)\($w\)$s[:-]$s\(.*\)$s\$|\1$fs\2$fs\3|p" |
        awk -F"$fs" '{
            indent = length($1)/2;
            if (length($2) == 0) { conj[indent]="+";} else {conj[indent]="";}
            vname[indent] = $2;
            for (i in vname) {if (i > indent) {delete vname[i]}}
                if (length($3) > 0) {
                    vn=""; for (i=0; i<indent; i++) {vn=(vn)(vname[i])("_")}
                    printf("%s%s%s%s=(\"%s\")\n", "'"$prefix"'",vn, $2, conj[indent-1],$3);
                }
            }' |
        sed 's/_=/+=/g'
    ) < "$yaml_file"
}

yaml_load() {
    local yaml_file="$1"
    local yaml_prefix="$2"
    eval "$(yaml_parse "$yaml_file" "$yaml_prefix")"
}

# CONFIG

config_load() {
    if [ -e "config.yml" ]
    then
        yaml_load "config.yml" "CONFIG_"
    fi
    if [ -e "local.yml" ]
    then
        yaml_load "local.yml" "CONFIG_local_"
    fi
}

config_dump() {
    if [ -e "config.yml" ]
    then
        yaml_parse "config.yml" "CONFIG_"
    fi
    if [ -e "local.yml" ]
    then
        yaml_parse "local.yml" "CONFIG_local_"
    fi
}

# MODULES

declare -a MODULES_LOADED
MODULES_LOADED=("b5")

module_exists() {
    if [ -z "$1" ]
    then
        echo "Usage: module_exists <module_name>"
        return 254
    fi
    if [ -e "${MODULES_PATH}/${1}.sh" ]
    then
        return 0
    else
        return 1
    fi
}

module_load() {
    if [[ " ${MODULES_LOADED[*]} " == *" $1 "* ]]
    then
        return 0
    fi
    if ( module_exists "$1" )
    then
        source "${MODULES_PATH}/${1}.sh" && \
        MODULES_LOADED+=("$1")
    else
        echo -e "${B5_FONT_RED}Module $1 not found${B5_FONT_RESTORE}"
        echo -e "${B5_FONT_RED}ABORTING!${B5_FONT_RESTORE}"
        exit 1
    fi
}

module_all() {
    if [ -z "$1" ]
    then
        echo "Usage: module_all <module_action>"
        return 254
    fi
    for module in ${MODULES_LOADED[@]}
    do
        echo "Calling ${module}_$1" && \
            "${module}_$1" && \
            echo || return 1
    done
}
